local VERSION = tonumber(("$Revision: 43130 $"):match("%d+"))

local Parrot = Parrot
local Parrot_AnimationStyles = Parrot:NewModule("AnimationStyles")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-07 20:41:08 -0400 (Sat, 07 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = Parrot:L("Parrot_AnimationStyles")

local animationStyles = {}
local animationStylesCleanup = {}
local animationStylesValidate = {}
local animationStylesOverlap = {}

function Parrot_AnimationStyles:RegisterAnimationStyle(name, localName, func, cleanup, overlap)
	self = Parrot_AnimationStyles -- so people can Parrot:RegisterAnimationStyle
	AceLibrary.argCheck(self, name, 2, "string")
	AceLibrary.argCheck(self, localName, 3, "string")
	AceLibrary.argCheck(self, func, 4, "function")
	AceLibrary.argCheck(self, cleanup, 5, "function", "nil")
	AceLibrary.argCheck(self, overlap, 6, "boolean", "nil")
	if animationStyles[name] then
		error(("Animation style %q already registered"):format(name), 2)
	end
	animationStyles[name] = func
	animationStylesValidate[name] = localName
	animationStylesCleanup[name] = cleanup
	animationStylesOverlap[name] = overlap or false
end
Parrot.RegisterAnimationStyle = Parrot_AnimationStyles.RegisterAnimationStyle

function Parrot_AnimationStyles:HasAnimationStyle(name)
	self = Parrot_AnimationStyles -- so people can Parrot:RegisterAnimationStyle
	AceLibrary.argCheck(self, name, 2, "string")
	return not not animationStyles[name]
end
Parrot.HasAnimationStyle = Parrot_AnimationStyles.HasAnimationStyle

function Parrot_AnimationStyles:GetAnimationStyle(name)
	local func = animationStyles[name]
	if func then
		return func, animationStylesCleanup[name], animationStylesOverlap[name]
	else
		return animationStyles.Straight, animationStylesCleanup.Straight, animationStylesOverlap.Straight
	end
end

function Parrot_AnimationStyles:GetAnimationStylesValidate(name)
	return animationStylesValidate
end

------------------------------------------------------------------------------
-- Animation style registry --------------------------------------------------
------------------------------------------------------------------------------

Parrot_AnimationStyles:RegisterAnimationStyle("Straight", L["Straight"], function(fontstring, scrollArea, percent)
	local fsHeight = fontstring:GetHeight()
	local height = scrollArea.height - fsHeight
	local bottom = scrollArea.yOffset
	local center = scrollArea.xOffset
	if scrollArea.reverse then
		percent = 1 - percent
	end
	local y = bottom + percent * height
	local x = center
	local align = scrollArea.align
	local point
	if align == "LEFT" then
		point = "BOTTOMLEFT"
	elseif align == "RIGHT" then
		point = "BOTTOMRIGHT"
	else -- center
		point = "BOTTOM"
	end
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, nil, true)
Parrot_AnimationStyles:RegisterAnimationStyle("Left parabola", L["Left parabola"], function(fontstring, scrollArea, percent)
	local fsHeight = fontstring:GetHeight()
	local height = scrollArea.height - fsHeight
	local bottom = scrollArea.yOffset
	local center = scrollArea.xOffset
	if scrollArea.reverse then
		percent = 1 - percent
	end
	local y = bottom + percent * height
	local x = center
	local align = scrollArea.align
	local point
	if align == "LEFT" then
		point = "BOTTOMLEFT"
	elseif align == "RIGHT" then
		point = "BOTTOMRIGHT"
	else -- center
		point = "BOTTOM"
	end
	x = x - (1 - (percent - 0.5)^2) * height * 1/2
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, nil, true)
Parrot_AnimationStyles:RegisterAnimationStyle("Right parabola", L["Right parabola"], function(fontstring, scrollArea, percent)
	local fsHeight = fontstring:GetHeight()
	local height = scrollArea.height - fsHeight
	local bottom = scrollArea.yOffset
	local center = scrollArea.xOffset
	if scrollArea.reverse then
		percent = 1 - percent
	end
	local y = bottom + percent * height
	local x = center
	local align = scrollArea.align
	local point
	if align == "LEFT" then
		point = "BOTTOMLEFT"
	elseif align == "RIGHT" then
		point = "BOTTOMRIGHT"
	else -- center
		point = "BOTTOM"
	end
	x = x + (1 - (percent - 0.5)^2) * height * 1/2
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, nil, true)

Parrot_AnimationStyles:RegisterAnimationStyle("Pow", L["Pow"], function(fontstring, scrollArea, percent, num, max)
	local fsHeight = fontstring:GetHeight()
	local middle = scrollArea.yOffset + scrollArea.height / 2
	local center = scrollArea.xOffset
	local reverse = scrollArea.reverse
	local topMiddle
	local topDiff = (max - 1) * 0.5 * fsHeight
	local currDiff = -(num - 1) * fsHeight
	local y
	if not reverse then
		y = middle + topDiff + currDiff
	else
		y = middle - topDiff - currDiff
	end
	
	local x = center
	local point = scrollArea.align
	-- now for squiggle
	local time = GetTime()
	if not fontstring.nextSquiggle or fontstring.nextSquiggle < time then
		fontstring.nextSquiggle = time + 0.05
		fontstring.squiggleX = math.random(-1, 1)
		fontstring.squiggleY = math.random(-1, 1)
	end	
	x = x + fontstring.squiggleX
	y = y + fontstring.squiggleY
	
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, function(fontstring, scrollArea)
	fontstring.nextSquiggle = nil
	fontstring.squiggleX = nil
	fontstring.squiggleY = nil
end, false)
